import json

import boto3
import pytest
from botocore.exceptions import ClientError

from aws_sm import SecretsManager


class TestSecretsManager:
    @pytest.fixture(autouse=True)
    def setUp(self):
        self.secret_name = "secret_name"
        self.secretsmanager = SecretsManager(
            "ap-northeast-1", "aws_acccess_key_id", "aws_secrets_key"
        )

    def test_basic_case_get_secret_value(self):
        data = {"SECRET_KEY": "secret_value", "ACCESS_KEY": "access_key"}

        res = self.secretsmanager.get_secret_value("SECRET_KEY", data)
        assert res == "secret_value"

    def test_nonexistent_secret_key_case_get_secret_value(self):
        data = {"SECRET_KEY": "secret_value", "ACCESS_KEY": "access_key"}

        with pytest.raises(KeyError):
            self.secretsmanager.get_secret_value("XXXXXXX", data)

    def test_basic_case_get_secret_values(self, mocker):
        data = {"SECRET_KEY": "secret_value"}
        mock_client = mocker.patch.object(boto3, "client")
        mock_client().get_secret_value.return_value = {"SecretString": json.dumps(data)}

        res = self.secretsmanager.get_secret_values(self.secret_name)
        assert res == data

    def test_different_secret_access_key_case_get_secret_values(self):
        secretsmanager = SecretsManager(
            "ap-northeast-1", "aws_acccess_key_id", "different_aws_secrets_key"
        )
        with pytest.raises(ClientError):
            secretsmanager.get_secret_values(self.secret_name)

    def test_incorrect_response_format_case_get_secret_values(self, mocker):
        mock_client = mocker.patch.object(boto3, "client")
        mock_client().get_secret_value.return_value = {
            "non-SecretString": {"SECRET_KEY": "secret_value"}
        }
        with pytest.raises(KeyError):
            self.secretsmanager.get_secret_values(self.secret_name)

    def test_json_decoding_fails_case_get_secret_values(self, mocker):
        data = "string response"
        mock_client = mocker.patch.object(boto3, "client")
        mock_client().get_secret_value.return_value = {"SecretString": data}

        with pytest.raises(json.JSONDecodeError):
            self.secretsmanager.get_secret_values(self.secret_name)



import json
import logging

import boto3
from botocore.exceptions import ClientError

logger = logging.getLogger(__name__)


class SecretsManager:
    def __init__(self, aws_region, aws_access_key_id=None, aws_secret_access_key=None):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.aws_region = aws_region

    def get_secret_values(self, secret_name):
        try:
            client = boto3.client(
                "secretsmanager",
                aws_access_key_id=self.aws_access_key_id,
                aws_secret_access_key=self.aws_secret_access_key,
                region_name=self.aws_region,
            )
            resp = client.get_secret_value(SecretId=secret_name)
            secret_values = json.loads(resp["SecretString"])
        except ClientError:
            logger.exception(
                f"[Secrets Manager botocore.exceptions.ClientError] secret_name={secret_name}"
            )
            raise
        except KeyError:
            logger.exception(
                f"[Secrets Manager KeyError] incorrect response format. secret_name={secret_name}"
            )
            raise
        except json.JSONDecodeError:
            logger.exception(
                f"[Secrets Manager ValueError] JSON decoding fails secret_name={secret_name}"
            )
            raise
        else:
            return secret_values

    def get_secret_value(self, secret_key, secret_values):
        try:
            secret = secret_values[secret_key]
        except KeyError:
            logger.exception(
                f"[Secrets Manager KeyError] secret_key does not exists. secret_key={secret_key}"
            )
            raise
        else:
            return secret
